/*
 *******************************************************************************
 * Nom du projet : THC_HAL_MTTI
 * Fichier       : THC_HAL_MTTI.ino
 * Auteur        : MTernisien-MTTI (www.mt-ti.fr)
 * Date          : 08/02/2025
 * Version       : 0.3
 * Description   : Permet la mesure de la tension d'arc (divisée) puis l'envoie avec filtrage au script Python sur LinuxCNC
 *
 * Matériel utilisé :
 * - Arduino UNO
 *
 * Fonctionnalités :
 * - Mesure la tension sur A0 
 * - Filtre en ajoutant 10ù de la mesure precedente à la nouvelle
 * - Envoie la valeur filtrée sur le port serie
 * - Reste a modifier le code afin d'optimiser le filtrage de la valeur mesurée durant les 200ms
 *
 * Historique des modifications :
 * - v0.1: Creation initiale, envoi de la valeur toutes les 50ms, non filtré
 * - v0.2: Mise en place du filtrage de la valeur
 * - v0.3: Modification du delai à 200ms, HAL etait saturé et le traitement de la valeur etait saccadé
 * - v1: (A venir) Ajout d'une filtration optimisée, mettre en place un DEFINE pour selectionner le "conversionFactor" selon la configuration du pont diviseur et du convertisseur
 *
 *******************************************************************************
 */



// Configuration
const int arcVoltagePin = A0;      // Broche où est connectée la mesure de tension
const unsigned long updateInterval = 200; // Intervalle d'envoi (200ms pour ne pas saturer HAL)

// Facteur de conversion corrigé
const float conversionFactor = 1000.0 / 1023.0; // Coefficient adapté pour 50:1 + conditionneur 0-20V → 0-5V

unsigned long lastUpdateTime = 0;  // Pour l'envoi régulier des valeurs
float stabilizedVoltage = 0;       // Valeur filtrée

void setup() {
  Serial.begin(115200);  
  analogReference(DEFAULT); // Référence 5V
}

void loop() {
  // Lecture instantanée de l'ADC
  int rawReading = analogRead(arcVoltagePin);
  float newVoltage = rawReading * conversionFactor; // Conversion en volts

  // Filtrage exponentiel (90% de la nouvelle valeur, 10% de l’ancienne)
  stabilizedVoltage = (stabilizedVoltage * 0.1) + (newVoltage * 0.9);

  // Envoi de la valeur filtrée
  unsigned long currentTime = millis();
  if (currentTime - lastUpdateTime >= updateInterval) {
    lastUpdateTime = currentTime;
    Serial.println((int)stabilizedVoltage); // Envoi sans décimales
  }
}
